// Prevent console window in addition to Slint window in Windows release builds when, e.g., starting the app via file manager. Ignored on other platforms.
#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

mod gui;
use gui::GUIElements;
use rfd::{FileDialog, MessageDialog};
use std::{cell::RefCell, rc::Rc};
use slint::SharedString;

slint::include_modules!();

fn main() -> Result<(), slint::PlatformError> {
    let main_window  = Rc::new(RefCell::new(MainWindow::new()?));
    let gui_elements = Rc::new(RefCell::new(GUIElements::new(main_window.clone())));
    let main_window  = main_window.borrow();

    main_window.set_main_tab_vram_bg(GUIElements::create_bg()?);

    let gui_elements_cloned = gui_elements.clone();
    main_window.on_main_tab_add_file_clicked(move || {
        let mut gui_elements = gui_elements_cloned.borrow_mut();
        let     file         = FileDialog::new()
            .add_filter("Images", &["png", "bmp", "jpeg"])
            .add_filter("All",    &[""])
            .set_title("Add TIM image")
            .pick_file();

        if let Some(file) = file {
            if let Result::Err(error) = gui_elements.add_new_vram_file(file) {
                MessageDialog::new().set_title("Loading Image failed").set_level(rfd::MessageLevel::Error).set_description(error.to_string()).show();
            }
        }
    });

    let gui_elements_cloned = gui_elements.clone();
    main_window.on_main_tab_remove_file_clicked(move |idx: i32| {
        let mut gui_elements = gui_elements_cloned.borrow_mut();
        if idx >= 0 {
            gui_elements.remove_vram_file(idx as usize);
        }
    });

    let gui_elements_cloned = gui_elements.clone();
    main_window.on_move_vram_image(move |idx: i32, dx: i32, dy: i32| {
        let mut gui_elements = gui_elements_cloned.borrow_mut();
        gui_elements.move_vram_image(idx as usize, dx, dy);
    });

    let gui_elements_cloned = gui_elements.clone();
    main_window.on_file_tab_browse_convert_image(move || {
        let gui_elements = gui_elements_cloned.borrow_mut();
        let file         = FileDialog::new()
            .add_filter("Images", &["png", "bmp", "jpeg"])
            .add_filter("All",    &[""])
            .set_title("Select image file")
            .pick_file();

        if let Some(file) = file {
            let main_window = gui_elements.main_window.borrow();
            if let Some(file_path) = file.to_str() {
                main_window.set_file_tab_browse_path(SharedString::from(file_path));
            }
        }
    });

    main_window.run()
}